;****************************************************************
;		HandyAudition Music File Receiver		*
;****************************************************************
; All of these functions use the following ZP Vars:		*
;	-RxActive						*
;	-RxLastFrame						*
;	-RxLastByte						*
;	-RxLastStat						*
;	-RxErrored						*
;	-RxTooMuch						*
;	-RxOff_Lo,Hi						*
;****************************************************************
RX_MUSIC_MAXSIZE	EQU $1000
;****************************************************************
; InitMusicRx::							*
;	Initialize the UART in 9N1 mode at 9600bps, receive	*
; only. Note that we'll be ignoring the 9th bit, so this is	*
; equivalent to 8N2 for the host.				*
;****************************************************************
InitMusicRx::
	STZ RxActive		; Ensure Rx's start as inactive, then
	STZ RxErrored		; premptively set up our IRQ Handler.
	STZ RxTooMuch
	SETIRQVEC 4,MusicRx_Handler

	LDA #12			; Timer 4 Divider (UART Clock) = (1MHz / 9615) / 8
	STA $FD10		; UART Hardware requires 8x bps for synchro.
	LDA #%00011000
	STA $FD11		; Timer 4 is now running @ 1MHz / 9615

	LDA#%01001101		; Only enable receive IRQs, 9N1 Mode,
	STA $FD8C		; Clear Errors, Open-Collector Pin.
	rts
;****************************************************************
; MusicRx_Handler::						*
;	Receive interrupt handler, overwrites the current music	*
; track with our newly received one.				*
;****************************************************************
MusicRx_Handler::
	PHA
	LDA $FD8C
	STA RxLastStat
	LDA $FD8D
	STA RxLastByte
	STA $FDB0		; Color Key 0 BR reflects current byte

	LDA FrameCounter
	STA RxLastFrame

	LDA#%00001101		; Rewrite the mode register with Rx IRQs disabled
	STA $FD8C
	LDA#%00010000		; Then re-ack the UART interrupt
	STA $FD80		; (WHAT?! It hasn't risen yet?!)
	LDA#%01001101
	STA $FD8C		; Now we can re-enable Rx IRQs

	LDA RxLastStat
	AND#%00001100		; If we've gotten an overrun or framing
	beq .noErrors		; error, mark the current stream as corrupt
	LDA#$FF			; so we can ignore any future data and
	STA RxErrored		; prevent the user from playing the track.
	STA RxActive
	STZ RxTooMuch
	PLA
	rts
.noErrors
	LDA RxLastStat
	AND#%01000000
	bne .gotByte
	PLA
	rts
.gotByte
	LDA RxActive		; Check to see if this is a continuing receive
	bne .continueRec	; or the first byte of a track.
	LDA#$FF
	STA RxActive
	STZ RxErrored
	STZ RxTooMuch

	LDA#<HandyMusic_Song_BaseAddress
	STA RxOff_Lo
	LDA#>HandyMusic_Song_BaseAddress
	STA RxOff_Hi

	PHX
	PHY
	jsr HandyMusic_StopAll
	PLY
	PLX

.continueRec
	LDA RxErrored		; Drop the rest of the data if we've
	ORA RxTooMuch		; encountered an error or received more
	bne .done		; than RX_MUSIC_MAXSIZE

	LDA RxLastByte		; Store the next byte of our music, then
	STA (RxOff_Lo)		; increment our destination address and check
				; to make sure we're within the size limit
	INC RxOff_Lo
	bne .checkBounds
	INC RxOff_Hi
.checkBounds
	LDA RxOff_Hi
	CMP#>(HandyMusic_Song_BaseAddress + RX_MUSIC_MAXSIZE)
	bne .done
	LDA RxOff_Lo
	CMP#<(HandyMusic_Song_BaseAddress + RX_MUSIC_MAXSIZE)
	bne .done
	LDA#$FF
	STA RxTooMuch
.done
	PLA
	rts


;****************************************************************
; CheckRxDone::							*
;	Closes the current .mus receive operation if no data	*
; has been received within 20 frames (1/3 sec). Must be called	*
; once every vblank.						*
;****************************************************************
CheckRxDone::
	LDA RxActive
	bne .checkTimeout
	rts
.checkTimeout
	LDA FrameCounter
	SBC RxLastFrame
	CMP#20
	bpl .killReceive
	rts
.killReceive
	STZ RxActive
	STZ $FDA0		; We also need to clear color key 0, as it will be altered
	STZ $FDB0		; during the receive to show the current byte's contents
	rts
